#include <iostream>
#include <cmath>
#include "globals.h"

using namespace std;
unsigned int WIN_SIZE = 800;                // Window dimensions
GLuint mesh_list;  // Display list used to draw everything
GLint p1_attrib;   // Vertex attribute
GLint p2_attrib;   // Vertex attribute

/* Draw a triangle. Since for each vertex we need to compute the distance 
to the opposite line, we pass the two other vertices as attributes. */
void draw_triangle(float x0, float y0, float z0, 
                   float x1, float y1, float z1,
                   float x2, float y2, float z2)
{
  glVertexAttrib4fARB(p1_attrib, x1, y1, z1, 1);
  glVertexAttrib4fARB(p2_attrib, x2, y2, z2, 1);
  glVertex4f(x0, y0, z0, 0);  
  
  glVertexAttrib4fARB(p1_attrib, x2, y2, z2, 1);
  glVertexAttrib4fARB(p2_attrib, x0, y0, z0, 1);
  glVertex4f(x1, y1, z1, 1);    
  
  glVertexAttrib4fARB(p1_attrib, x0, y0, z0, 1);
  glVertexAttrib4fARB(p2_attrib, x1, y1, z1, 1);
  glVertex4f(x2, y2, z2, 2);  
}

/* Setup for single pass wireframe rendering. This function installs the 
required shaders, sets up uniforms and compiles the entire rendering 
into a display list. */
void setup_single_pass()
{
  // Install the fragment and vertex programs
  GLhandleARB Prog = install_shaders();
  
  // Assign attribute locations and uniforms
  glUseProgramObjectARB(Prog);
  p1_attrib = glGetAttribLocationARB(Prog, "p1_3d");
  p2_attrib = glGetAttribLocationARB(Prog, "p2_3d");
  glUniform2fARB(glGetUniformLocationARB(Prog,"WIN_SCALE"), WIN_SIZE/2.0, WIN_SIZE/2.0);
  glUniform3fARB(glGetUniformLocationARB(Prog,"WIRE_COL"), 0.8,0.3,0.1);
  glUniform3fARB(glGetUniformLocationARB(Prog,"FILL_COL"), 0.7,0.8,0.9);
  
  mesh_list = glGenLists(1);
  
  // We do everything in a display list which is compiled below.
  glNewList(mesh_list, GL_COMPILE);
  glBegin(GL_TRIANGLES);
  
  for (int i = 0; i < 448; i ++) 
    {
    for (int x = 0; x < 200; x += 6) 
      {
      if (((unsigned int) teapot_geometry[i][x+6]) == 666)
        break;
      draw_triangle(teapot_geometry[i][x], teapot_geometry[i][x+1], teapot_geometry[i][x+2],
                    teapot_geometry[i][x+3], teapot_geometry[i][x+4], teapot_geometry[i][x+5],
                    teapot_geometry[i][x+9], teapot_geometry[i][x+10], teapot_geometry[i][x+11]);
      draw_triangle(teapot_geometry[i][x], teapot_geometry[i][x+1], teapot_geometry[i][x+2],
                    teapot_geometry[i][x+9], teapot_geometry[i][x+10], teapot_geometry[i][x+11],
                    teapot_geometry[i][x+6], teapot_geometry[i][x+7], teapot_geometry[i][x+8]);
      }
    }      
  glEnd();      
  glEndList();
}

void idle()
{
  glutPostRedisplay();
}

void display()
{
  // A primitive time variable used for rotation
  static float T = 0.0;
  T += 0.001;
  
  // Clear buffers, and set up modelview matrix
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(6*cos(T),4*sin(T),4,0,0,2,0,0,1);
  glCallList(mesh_list);
  glutSwapBuffers();
}



int main (int argc, char * argv[]) 
{
  // GLUT INIT
  glutInitDisplayMode(GLUT_RGBA|GLUT_DOUBLE|GLUT_DEPTH|GLUT_ALPHA);
  glutInitWindowSize(WIN_SIZE, WIN_SIZE);
  glutInit(&argc, argv);
  glutCreateWindow("OBJ Viewer");
  glutDisplayFunc(display);
  glutIdleFunc(idle);

#ifdef USING_GLEW
	glewInit();
#endif
  
  //GL INIT
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.8,0.7,0.6,0);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(53,1.0,0.05,20);
  
  // Setup for wireframe drawing
  setup_single_pass();
  
  glutMainLoop();
  return 0;
}
